@@ -13,12 +13,14 @@ from .io import HDF5Dataset
 from .io import load_dataset
 
 from .utils import readImg
+from .utils import compute_mean
 from .utils import saveNumpy
 from .utils import train_val_split
+from .utils import extract_frames
 from .utils import plotAcc 
 from .utils import url_to_image 
 from .utils import toMatplotlib 
-from .utils import translate 
+from .utils import translate
 from .utils import rotate 
 from .utils import rotate_bound 
 from .utils import resize 
