@@ -2,25 +2,28 @@
 # Github: http://www.github.com/jasmcaus
 
 # Importing the necessary packages
+import sys
+sys.path.append('..')
+
 import os
 import time
 import numpy as np
 from .utils import readImg
 from .utils import saveNumpy
+from .preprocessing import MeanProcess
 
-def preprocess_from_directory(DIR, classes, name, channels=1, IMG_SIZE=224, train_size=None, isNormalize=False, isShuffle=True, isSave = True, display_count=True):
+def preprocess_from_directory(DIR, classes, name=None, channels=1, IMG_SIZE=224, train_size=None, isNormalize=False, mean_subtraction=None, isShuffle=True, save_train = True, display_count=True):
     """
     Reads Images in base directory DIR using 'classes' 
     Returns
         train -> Image Pixel Values with corresponding labels
-    Saves the above variables as .npy files if isSave = True
+    Saves the above variables as .npy files if save_train = True
     """
 
     train = [] 
     try:
-        if isSave is True and not ('.npy' in name or '.npz' in name):
-            print('[ERROR] Specify the correct numpy destination file extension (.npy or .npz)')
-            raise TypeError
+        if save_train is True and not ('.npy' in name or '.npz' in name):
+            raise TypeError('[ERROR] Specify the correct numpy destination file extension (.npy or .npz)', name)
             
         elif os.path.exists(name):
             since = time.time()
@@ -29,37 +32,46 @@ def preprocess_from_directory(DIR, classes, name, channels=1, IMG_SIZE=224, trai
             end = time.time()
             print('[INFO] Loaded in {:.0f}s from Numpy Files'.format(end-since))
 
-            # Raising TypeError although not a TypeError to escape from the try block
-            # Alternatively, use the 'pass' keyword
-            # raise TypeError
-
             return train
 
         else:
             since_preprocess = time.time()
             print(f'[INFO] Could not find {name}. Generating the Image Files')
 
+            if not save_train:
+                name = None
+            if save_train and name is None:
+                raise ValueError('[ERROR] Specify a destination file name')
+            
             if train_size is None:
                 train_size = len(os.listdir(os.path.join(DIR, classes[0])))
 
+            # Checking if 'mean_subtraction' values are valid. If yes, then mean subtraction is applied
+            subtract_mean = check_mean_subtraction(mean_subtraction, channels)
+
             for item in classes:
                 class_path = os.path.join(DIR, item)
-                classNum = classes.index(item)
+                class_label = classes.index(item)
                 count = 0 
                 for image in os.listdir(class_path):
                     if count != train_size:
                         image_path = os.path.join(class_path, image)
 
-                        # Returns image RESIZED and GRAY
-                        gray = readImg(image_path, IMG_SIZE=IMG_SIZE, channels=channels)
-                        if gray is None:
+                        # Returns image RESIZED and img
+                        img = readImg(image_path, IMG_SIZE=IMG_SIZE, channels=channels)
+                        if img is None:
                             continue
                         # Normalizing
-                        if isNormalize is True:
-                            gray = normalize(gray)
+                        if isNormalize:
+                            img = normalize(img)
+                        
+                        if subtract_mean:
+                            mean_subtract = MeanProcess(mean_subtraction, channels)
+                            img = mean_subtract.mean_preprocess(img, channels)
                             
-                        train.append([gray, classNum])
+                        train.append([img, class_label])
                         count +=1 
+
                         if display_count is True:
                             _printTotal(count, item)
                     else:
@@ -73,27 +85,27 @@ def preprocess_from_directory(DIR, classes, name, channels=1, IMG_SIZE=224, trai
             train = np.array(train)
 
             # Saves the Train set as a .npy file
-            if isSave is True:
+            if save_train is True:
                 #Converts to Numpy and saves
-                if '.npy' in name:
+                if name.endswith('.npy'):
                     print('[INFO] Saving as .npy file')
-                elif '.npz' in name:
+                elif name.endswith('.npz'):
                     print('[INFO] Saving as .npz file')
                 
                 since = time.time()
                 # Saving
                 saveNumpy(name, train)
-
                 end = time.time()
                 
                 time_elapsed = end-since
 
-                print('{} saved! Took {:.0f}m {:.0f}s'.format(name, time_elapsed // 60, time_elapsed % 60))
+                print('[INFO] {} saved! Took {:.0f}m {:.0f}s'.format(name, time_elapsed // 60, time_elapsed % 60))
 
             #Returns Training Set
             end_preprocess = time.time()
             time_elapsed_preprocess = end_preprocess-since_preprocess
-            print('Preprocessing complete! Took {:.0f}m {:.0f}s'.format(time_elapsed_preprocess // 60, time_elapsed_preprocess % 60))
+            print('----------------------------------------------')
+            print('[INFO] Preprocessing complete! Took {:.0f}m {:.0f}s'.format(time_elapsed_preprocess // 60, time_elapsed_preprocess % 60))
 
             return train
 
@@ -103,6 +115,14 @@ def preprocess_from_directory(DIR, classes, name, channels=1, IMG_SIZE=224, trai
 def _printTotal(count, category):
     print(f'{count} - {category}')
 
+def check_mean_subtraction(value, channels):
+    if value is None:
+        return False
+    elif type(value) is tuple and len(value) == channels:
+        return True
+    else:
+        raise ValueError(f'[ERROR] Expected a tuple of dimension {channels}', value) 
+
 def shuffle(train):
     """
     Shuffles the Array
@@ -140,6 +160,6 @@ def normalize(x):
     # x/=255.0 raises a TypeError
     # x = x/255.0
     
-    # Converting to float32 and normalizing
+    # Converting to float32 and normalizing (float32 saves memory)
     x = x.astype('float32') / 255
     return x
\ No newline at end of file
